;**************************************************************************
; RADIOTMR.ASM
;
; PROGRAM: Timer to start AWIA mini-disc AM-F5 recorder. 
;
; DESCRIPTION: turns on for the Radio 4 Shipping forecasts.
; AUTHOR:  Douglas Rice
; Copyright 2000
;
;	This code is a cut and paste from the LED and sound version
;	Where code is not required it has been commented out using IFDEF
;
;	The Unit turns on or off the AWIA mini-disc recorder at one of the eight times.
;	
;	The recorder's remote control has push buttons with a series resistance.
;	To start the recording requires a sequence of button presses.	
;
;	To start recording press and release:
;		Over about 4 seconds:
;		REC/EDIT
;		REC/EDIT + PLAY
;		release all buttons
;		PLAY
;
;	To stop recording press and release:
;		STOP
;	
;
;	The PIC16C84 chip stores the ON OFF times in BCD in the PIC EEPROM.
;
;	Usage: Two Buttons - <SET> and <NEXT> short to GND on PRESS
;
;	Normally only <NEXT> needs to be pressed. 
;	<SET> is only required to check or set the time
;
;	If an RS232 terminal is connected to PortB pin 0, via a resistor
;	Sending a 0 simulates pressing the NEXT button.
;	Sending a 1 simulates pressing the SET  button  
;
;
;	There are two modes:- Normal use and Check/Set Mode.
;	The <NEXT> button Cycles around the ON and OFF states. 
;
;	The Check/Set mode is entered by pressing the <SET> button
; 	
;	<SET> sets the digit up and then is used to set the time.
;	<NEXT> is used to move onto the next digit or state.
;	 
;	For the LCD based version:
;	==========================
;	LCD display	comment	
;	===========	=======
;	t 22:44 ?	entering Time
;	L FF:FF ?	showing a logged time
;	1-00:00 ?	1st Off time
;	2+00:00 ?	1st On  time
;	3-00:00 ?	2nd Off time
;	4+00:00 ?	2nd On  time
;	5-00:00 ?	3rd Off time
;	6+00:00 ?	3rd On  time
;			there are 8 on and 8 off times.
;
;	So if <SET> is pressed by mistake, keep pressing <NEXT>
;
;	To set the time enter the value of each digit by repeatedly pressing 
;	the <SET> button to increment the digit from zero. 
; 	
;
;	Checking the ON OFF times.
;	The list of ON OF times is stored in the EEPROM
;	The values are stored in BCD notation. There is a list of 8 ON and OFF times
;	
;	The RS232 output outputs at 1200 baud. Connect to RS232 without inversion.
; 
;	when setting the time, the display looks like:
;	#0 now:11:59		- current time 
;	#1 On@:11:59		- Turn On  power @11:59
;	#2Off@:12:05		- Turn Off power @12:05
;
;	To change the time:
;	press <NEXT> and the H0 digit appears, and press <SET> to inc H0
;	#1 On@:11:59 1		- display entered value of H0
;
;	press <NEXT> and the 0H digit appears, and press <SET> to inc 0H
;	#1 On@:11:59 13		- display entered value of 0H
;		
;	press <NEXT> and the M0 digit appears, and press <SET> to inc M0
;	#1 On@:11:59 13:1	- display entered value of M0
;
;	press <NEXT> and the 0M digit appears, and press <SET> to inc 0M
;	#1 On@:11:59 13:14	- display entered value of 0M
;		
;	press <NEXT> and set appears, 
;	#1 On@:11:59 13:14set	- press <SET> to accept, <NEXT> to cancel.
;
;	The setting mode is left.
;
;	When not in the setting mode, <NEXT> toggles radio On with beeps
;	On each second, the Serial Port outputs a  display is  tHH:MM:SS:status 
;	e.g. t21:34:45:Off 
;	or   t21:34:45:On beep 
;	or   t00:15:01:On        
;
;	To record the forecast, set the time to real time.
;	Set the Cassette Recorder to record Radio 4.
;	Toggle Radio Cassette to OFF and wait.
;
;	After forecast has been recorded, Toggle on tape and rewind and listen, then
;	set up to record the radio again.
;
;
;	Crystal:	32kHz
;
;	I/O used:-
;	PORTA:0	pulled low during On, Hi-Z during off.
;	PORTA:1 BiColour LED GREEN
;	PORTA:2 BiColour LED RED 
;	PORTA:2 AWIA STOP low via 20k to apply STOP
;	PORTA:3 AWIA Play low via 5k6 to apply Play
;	PORTA:4	AWIA RED/EDIT low via 12K to apply REC/PLAY
;
;	
;	PORTB:7	NC		
;	PORTB:6	NC
;	PORTB:5	IP: SET button SWITCH to  GND
;	PORTB:4	IP: NEXT  button SWITCH to  GND
;	PORTB:3	LCD pin
;	PORTB:2	LCD pin
;	PORTB:1	LCD pin
;	PORTB:0	RS232 input @ 1200 baud. 
;			Char input emulates button presses
;			If LSB is 1, emulate SET  pressed
;			If LSB is 0, emulate NEXT pressed
;
;	(Bi Colour LED) {set Button}
;			{NEXT Button }
;
;
	INCLUDE "p16F84.inc"       
;
;**************************************************************************

        LIST P=16F84, R=DEC
	__idlocs  0x1234
	__config  _XT_OSC&_PWRTE_ON & _WDT_OFF

	cblock 	0x0c
		; variables used for context saving 
		w_temp
		status_temp
	endc

;--------------------------------------------------------------------------
; Sec 0. Build Constants - uncomment to include in the code.
;--------------------------------------------------------------------------
; This file is a combination of RT001 and the routines for the SED1200 LCD
; The old LED, BEEP and Serial Out Routines were commented out using IFDEF
; Currently, the code only builds for some of these below.
;


buildLCD		; Build LCD code
;buildSRwrt		; Build Serial Output code
;buildIntrtn		; Interupt routine
buildSRwrt		; Enable serial out.
buildMacroRS232		; Build RS232 code that uses a macro

;buildLCDsed1200
buildLCD_HD44780


;--------------------------------------------------------------------------
; Sec 1. Equates and Constants
;--------------------------------------------------------------------------


		; The General Purpose Registers start at the end of the 
		; Special Purpose Registers.


; IPMentState values
; in normal running or in time setting modes
;
SET_NORM	EQU	0
SET_SET	EQU	2

; STstate flag to indicate if beep is enabled.
; STstate is a variable of flag bits
STrelay		EQU	0
STbeep		EQU	1


; DoTimeSlice bits
; these bits are set to schedule a timer chain event
DTsS4event	EQU	0	; 
DTsSevent	EQU	1
DTsMevent	EQU	2
DTsHevent	EQU	3
DTopenedevent	EQU	4
DTSRwrtEvent	EQU	5

;--------------------------------------------------------------------------
; Sec 1.1 Button and LED 
;--------------------------------------------------------------------------

BUTTON_SET	EQU	5
BUTTON_NEXT	EQU	4
BUTTON_ONOFF	EQU	6
BUTTON_OPENED	EQU	7	; If change then record time

;--------------------------------------------------------------------------
; Sec 1.2 Button and LED 
;--------------------------------------------------------------------------

; PORTA binary values
; some port A bits are set in AWAI.inc


PORT_RS232	EQU	PORTA
RS232OUT	EQU	0

;--------------------------------------------------------------------------
; Sec 1.3 Table Macro
;--------------------------------------------------------------------------


TEST_STRADDLE	MACRO	START
	if high( $ ) != high( START ) 
	    Error "Table straddles Page Boundary near " START   " "
	endif
	endm


;--------------------------------------------------------------------------
; Sec 2.0 Variables
;--------------------------------------------------------------------------
; Variables start 0x0C
;
; Variables for FlashLED ( FL ) process 
;

		;	DoTimeSlice Bits, to schedule, set bit
		;	0	run 	onQtrSecondTick
		;	1	run	onSecondTick
		;	2	run	onMinuteTick
		;	3	run	onHourTick
		;	4	not used
		;	5	run	Serial Port readout
		;	6	BeepPause
		;	7	run	BeepMachine


		cblock 	

DoTimeSlice

STstate	

CLtmr4sec	; counts quarter seconds

CLnbcdS 	; stores current time
CLnbcdM 
CLnbcdH 

CLdbcdM 	; stores the time being input
CLdbcdH 

CLbbcdM 	; stores the time being beeped out
CLbbcdH 
CLTNum		; Index into EEPROM when updating an ON/OFF time

CLTNumChk	; EEPROM index when testing for ON/OFF time

IPlast		; Input variables
IPnew		
IPbuttonEvent	
IPsetState	

SRout 		; for RS232 routines

SRin	; Used in an interupt routine for serial Input
SRcnt		
SRloopCnt

DispState	; Display State
OPcnt		; Count 
OPopcnt		; Count 

AWstate		; AIWA minidisc contral state machine
BILEDPORT	; park Bi color led off PORTA.
PORTBtemp
	;
		ENDC


OPbufferStart	EQU	0x30	; 32 bytes of files that can store Dates

;--------------------------------------------------------------------------
; Sec 3. Program Code
;--------------------------------------------------------------------------

                ORG     0       
                GOTO    Start

		ORG 	4
		GOTO 	Intrtn

;--------------------------------------------------------------------------
; Sec 3.1	Main Program Init Code
;--------------------------------------------------------------------------

Start

		CLRF	INTCON
		; set up for 8 bits of serial in.
		MOVLW	8
		MOVWF	SRcnt

		; Set up PortB:<1:3> as outputs
		BSF	STATUS, RP0
		MOVLW	0XF1
		MOVWF	TRISB
		MOVLW	0xE0	; Make PORTA 4:0 outputs
		MOVWF	TRISA
		BCF	STATUS, RP0
		; clear all pending events
		CLRF	DoTimeSlice	
		; Clear the button inputs 
		CALL	IPtimeslice
		CLRF	IPbuttonEvent
		DECF	IPbuttonEvent,F
	
		CLRF	IPsetState
		CLRF	CLdbcdH
		CLRF	CLdbcdM
		CLRF	CLbbcdH
		CLRF	CLbbcdM
		CLRF	CLTNumChk

		MOVLW	0x0D	;CR
		CALL	SRtxChar	
		MOVLW	'*'
		CALL	SRtxChar	
		MOVLW	'U'
		CALL	SRtxChar	
		MOVLW	'*'
		CALL	SRtxChar	
		MOVLW	0x0D	;CR
		CALL	SRtxChar	
		MOVLW	0x0A	;CR
		CALL	SRtxChar	

		MOVLW	OPbufferStart
		MOVWF	FSR
		MOVLW	0x50-OPbufferStart
		MOVWF	OPcnt
		MOVLW	0xFF
Start1		
		MOVWF	INDF
		INCFSZ	FSR
		DECFSZ	OPcnt
		GOTO	Start1
		MOVLW	OPbufferStart
		MOVWF	OPcnt
		MOVWF	OPopcnt
Start2

		MOVLW	-2
		MOVWF	CLTNum

		CLRF	DispState

		; Change Prescaler
		CLRWDT
		; Configure Tmr 0
		BSF	STATUS,RP0
		; Set up prescaller for 8192 / 16 = 512 ticks perseconds.
		MOVLW	1 << INTEDG | 0x0 + 2 	; For Tmr0 0=/2, 1=/4, 2=/8,3=/16
		MOVWF	OPTION_REG
		BCF	STATUS,RP0

		MOVLW	0x0 
		MOVWF	CLnbcdS

		MOVLW	0x23
		MOVWF	CLnbcdH

		MOVLW	0x50
		MOVWF	CLnbcdM
		MOVLW	4
		MOVWF	CLtmr4sec

		CALL 	LCDstart
		
	; Make sure that the Serial OP does sometime.
		CALL	LCDredisp
		CLRF	STstate
		CALL	STtoggle

		; Enable Interupts
		MOVLW	1 << GIE | 1 << INTE
		MOVWF	INTCON

		GOTO	MainLoop

MLinitLoop	
		BTFSS	INTCON,T0IF
		GOTO	MLinitLoop
		; TMR0 timeout
		BCF	INTCON,T0IF
		DECFSZ 	LCDcnt,f               
		GOTO	MLinitLoop




;--------------------------------------------------------------------------
; Sec 3.2	Main Program 
;--------------------------------------------------------------------------

MainLoop	
	; Enable Interupts
	; Disable Interupts
MLdisableInt	
		CALL	CLtimeslice
;		CALL	AWts		; used to single step

		; Any events that exist
		BTFSC	DoTimeSlice,DTsS4event	; On 1/4 Second Tick
		CALL	SS4tick

		; Any events that exist
		BTFSC	DoTimeSlice,DTsSevent	; On Second Tick
		CALL	SStick

		BTFSC	DoTimeSlice,DTsMevent	; On Minute Tick
		CALL	SMtick

		BTFSC	DoTimeSlice,DTsHevent	; On Hour Tick
		CALL	SHtick

		BTFSC	DoTimeSlice,DTSRwrtEvent	; Serial Display
		CALL	SRwrtTime

		BTFSS	IPbuttonEvent,BUTTON_SET	; On BUTTON_SET Press
		CALL	IPsetPressed

		BTFSS	IPbuttonEvent,BUTTON_NEXT	; On BUTTON_NEXT Press
		CALL	IPnextPressed

		BTFSS	IPbuttonEvent,BUTTON_ONOFF	; On BUTTON_ONOFF Press
		CALL	IPonoffPressed

		BTFSS	IPbuttonEvent,BUTTON_OPENED	; On BUTTON_OPENED Press
		CALL	IPopenedPressed


		GOTO 	MainLoop
		

;--------------------------------------------------------------------------
; Sec 3.3 	Jump Table for running or time setting mode
;--------------------------------------------------------------------------


;--------------------------------------------------------------------------
; Sec 3.4	Sound out the digits using beeps
;--------------------------------------------------------------------------

; These routines are state machines that are used to beep out the time
; Initilize the two state machines. One is used to 

;--------------------------------------------------------------------------
; Sec 3.5	LCD display 
;--------------------------------------------------------------------------
; this code must be in the bottom page

IPsetJump
	;
	; W=1 for SET, 
	; W=0 from NEXT
	;
IPjmpTableLcd4Start
	ADDWF	IPsetState,W
	CLRF	PCLATH		; Assume that the tabel is in the bottom 256 byte
	ANDLW	0x03
	ADDWF	PCL,f
	; set choice State Table - limited to 2 states and two events
	GOTO	STtoggle	; SET:OFF,NEXT
	GOTO 	LCDInit		; SET:OFF,set
	GOTO	LCDOneDigit	; SET:YELLOW,NEXT
	GOTO	LCDNextDigit	; SET:YELLOW,set

	TEST_STRADDLE IPjmpTableLcd4Start

;--------------------------------------------------------------------------
; Sec 3.4	Display the digits on the LCD
;--------------------------------------------------------------------------

; These routines are state machines that are used to display on the LCD

LCDInit
	ifdef buildLCD
	; We run this from the toggle state and 
	; Prevent update of time display on second ticks.

	; copy the current time into the Display registers
	MOVFW	CLnbcdH
	MOVWF	CLdbcdH
	MOVFW	CLnbcdM
	MOVWF	CLdbcdM
	MOVLW	-2		; register number -2 for t, -1 for Logger
	MOVWF	CLTNum
;	; Clear the digit input registers
	CLRF	DispState
	GOTO	LCDredisp

LCDOneDigit
	; rewrite display and then position cursor
	; Are we to update Current time or ON/OFF time in EEPROM
		
	CALL	LCDredispTime
	; now move the cursor to the correct position.

	INCFSZ	DispState,F
	INCFSZ	DispState,F
	; Turn On Cursor and turn off Increment
	MOVLW	LCDCurOn
        CALL    LCDwrtCmd

	GOTO LCDredispCursor

LCDNextDigit
	; This increments the digit
	; DispState is incremented by the time this can be pressed


	MOVLW	high $
	MOVWF	PCLATH

	; divide the count by two
	RRF	DispState,W
	ADDWF	PCL,F
	GOTO	LCDNextGetTimer		; 2
	GOTO	LCDNextSetH0		; 4
	GOTO	LCDNextSet0H		; 6
	GOTO	LCDNextSetM0		; 8
	GOTO	LCDNextSet0M		; 10
	GOTO	LCDNextSet		; 12
	NOP
	NOP

	TEST_STRADDLE	LCDNextDigit

LCDNextGetTimer
	; Check CLTNum and if negative then its still logger
	BTFSS	CLTNum,7
	GOTO	LCDNextGetTimer0
	INCFSZ	CLTNum,f
	GOTO	LCDNextGetTimerLogged
	GOTO	LCDNextGetTimer0	; CLTNum is zero so get the value from the EEPROM

LCDNextGetTimerLogged
	MOVLW	0x99
	MOVWF	CLdbcdH
	MOVWF	CLdbcdM
	; We need to update the display
	GOTO	LCDredisp

	; get the next ON /OFF time from the EEROM for BEEP out
	; Overwrite the current time

LCDNextGetTimer0
	; read the next time from the EEPROM
	MOVFW	CLTNum
	ANDLW	0x1F
	CALL	EEread
	MOVWF	CLdbcdH

	; read 2nd digit
	INCF	CLTNum,f
	MOVFW	CLTNum
	ANDLW	0x1F
	CALL	EEread
	MOVWF	CLdbcdM
	INCF	CLTNum,W
	ANDLW	0x3F		; Restrict to 16 Locations or bytes each
	MOVWF	CLTNum
	; We need to update the display
	GOTO	LCDredisp

LCDNextSetH0
	; We have just
	MOVLW	0X10
	ADDWF	CLdbcdH,F
	MOVFW	CLdbcdH
	ADDLW	0xD0		; 
	BC	LCDNextSetH01
	GOTO	LCDredisp

LCDNextSetH01	
	; just clear both digits to save an instruction	
	CLRF	CLdbcdH
	GOTO	LCDredisp


LCDNextSet0H
	INCF	CLdbcdH,f	; this will never carry to M0
	MOVFW	CLdbcdH
	ADDLW	0x06		
	BDC	LCDNextSet0H2
	; check for 24 hours Allow 24:00 to be set so that the timer can be parked.
	MOVF	CLdbcdH,w
	ADDLW 0x00 - 0x25		
	SKPNZ	
	GOTO	LCDNextSet0H2		; its 24 so zero 0H digit
	GOTO	LCDredisp

LCDNextSet0H2				; unit counter carry to tens counter
	MOVLW	0xF0
	; digit has just incremented so windback
	ANDWF	CLdbcdH,F	; clear ls digit back to zero
	GOTO	LCDredisp

LCDNextSetM0
	MOVLW	0X10
	ADDWF	CLdbcdM,F
	MOVFW	CLdbcdM
	ADDLW	0xA0		
	BC	LCDNextSetM01
	; check for 60 minutes
	GOTO	LCDredisp


LCDNextSetM01		
	MOVLW	0x0F
	ANDWF	CLdbcdM,F
	GOTO	LCDredisp

LCDNextSet0M
	;	in (SET:SETH) inc pressed
	; Stay in same state
	;	in (SET:SETH) inc pressed
	; Stay in same state
	INCF	CLdbcdM,f
	MOVF	CLdbcdM,w
	ADDLW	0x06		
	BDC	LCDNextSet0M2
	GOTO	LCDredisp

LCDNextSet0M2		; unit counter carry to tens counter
	; gone from 9 to A, so zero digit
	MOVLW	0xF0
	ANDWF	CLdbcdM,F
	GOTO	LCDredisp

LCDredispTime
	; I want to display t,logged,on/off times
	; use CLTNum
	; 	-2	= t
	;	-1	= l
	;	0 to 1F = registers

	; display the updated registers.
	BTFSS	CLTNum,7	
	GOTO 	LCDredisp00

	; Now look for the

	; Position Cursor
	MOVLW	high $
	MOVWF	PCLATH
	MOVLW	2			; Cope with CLTNum starting with -2
	ADDWF	CLTNum,w
	ADDWF	PCL,F
	GOTO	LCDredispT	; display t
	GOTO	LCDredispL	; display L and next logged time

	TEST_STRADDLE	LCDredispTime


LCDredispT
        MOVLW   LCDCurRight0	;0xC0
        CALL    LCDwrtCmd
        MOVLW   't'
        CALL    LCDwrtChar
        MOVLW   ' '
        CALL    LCDwrtChar
	GOTO	LCDredispM

LCDredispL
        MOVLW   LCDCurRight0	;0xC0
        CALL    LCDwrtCmd
        MOVLW   'L'
        CALL    LCDwrtChar
        MOVLW   ' '
        CALL    LCDwrtChar
	; copy the last logged ON/OFF Time

	MOVFW	OPopcnt
	MOVWF	FSR
	; load byte from the buffer
	MOVFW	INDF
	MOVWF	CLdbcdH		; Store this for the serial display
	CALL 	LCDdispByte
	INCFSZ	OPopcnt,f
	MOVFW	OPopcnt
	MOVWF	FSR

	MOVLW	':'
        CALL    LCDwrtChar

	MOVFW	INDF
	MOVWF	CLdbcdM		; Store this for the serial display
	CALL 	LCDdispByte

	INCFSZ	OPopcnt,f
	MOVLW	-0x50
	ADDWF	OPopcnt,w
	SKPZ	
	GOTO 	LCDredispL0

	MOVLW	0x30
	MOVWF	OPopcnt

LCDredispL0	
	RETURN


LCDredisp00
        MOVLW   LCDCurRight0	;0xC0
        CALL    LCDwrtCmd

	RRF	CLTNum,w
	CALL	LCDwrtDecNibble
	; Arrows are not the same on both SED1200 and HD44780 so use
	MOVLW	'-'	;0xD2	; Up arrow
	BTFSC	CLTNum,1 	; test 2nd Bit
	MOVLW	'+'	;0xD3	; Down Arrow
	CALL	LCDwrtChar

LCDredispM
        MOVLW   LCDCurRight0+2	;0xC2
        CALL    LCDwrtCmd
	MOVFW	CLdbcdH
	CALL 	LCDdispByte
        MOVLW   ':'
        CALL    LCDwrtChar
	MOVFW	CLdbcdM
	CALL 	LCDdispByte

        MOVLW   ' '
        CALL    LCDwrtChar
        MOVLW   '?'
        CALL    LCDwrtChar
        MOVLW   ' '
        CALL    LCDwrtChar
	BSF	DoTimeSlice,DTSRwrtEvent	; Start Serial Display
	RETLW	SET_NORM		        ; Leave Setting States.

	RETURN

;	org 0x100
LCDredisp
	BSF	DoTimeSlice,DTSRwrtEvent	; Start Serial Display
	CALL	LCDredispTime
	; fall ino the next routine



LCDredispCursor

	; Position Cursor
	MOVLW	high $
	MOVWF	PCLATH
	MOVFW	DispState
	ADDWF	PCL,F

	MOVLW	LCDCurRight0	;0xC0			; Preamble
	GOTO	LCDredispCursorMove

	MOVLW	LCDCurRight0+2	;0xC2			; Hh:mm preamble
	GOTO	LCDredispCursorMove

	MOVLW	LCDCurRight0+3	;0xC3			; hH:mm preamble
	GOTO	LCDredispCursorMove

	MOVLW	LCDCurRight0+5	;0xC5			; hh:Mm preamble
	GOTO	LCDredispCursorMove

	MOVLW	LCDCurRight0+6	;0xC6			; hh:mM preamble
	GOTO	LCDredispCursorMove

	MOVLW	LCDCurRight0+8	;0xC8			; Set - this shall never be called though
	GOTO	LCDredispCursorMove		; Set   preamble, no BeepMachine req
	
	GOTO	LCDredispCursorLast		; Turn off BeepMachine and leave
	NOP

	TEST_STRADDLE	LCDredispCursor

LCDredispCursorMove
	; move the cursor
        CALL    LCDwrtCmd
	; Turn On Cursor and turn off Increment
	MOVLW	LCDCurOn
        CALL    LCDwrtCmd
	RETLW	SET_SET

LCDredispCursorLast
	MOVLW	-2
	MOVWF	CLTNum

	; Turn Off Cursor
	MOVLW	LCDCurOff
        CALL    LCDwrtCmd

	; Update the display with the current display
        MOVLW   LCDCurRight0	;0xC0
        CALL    LCDwrtCmd
        MOVLW   ' '
        CALL    LCDwrtChar
        MOVLW   ' '
        CALL    LCDwrtChar
	MOVFW	CLnbcdH
	CALL 	LCDdispByte
        MOVLW   ':'
        CALL    LCDwrtChar
	MOVFW	CLnbcdM
	CALL 	LCDdispByte
        MOVLW   ':'
        CALL    LCDwrtChar
	MOVFW	CLnbcdS
	CALL 	LCDdispByte
	;enable display update
	BSF	DoTimeSlice,DTSRwrtEvent	; Start Serial Display

	RETLW	SET_NORM		        ; Leave Setting States.



LCDNextSet
	; Are we to update Current time or ON/OFF time in EEPROM
	BTFSC	CLTNum,7	
	GOTO	LCDNextSet0
	; Update an EEPROM register
	; Write displayed HH:MM digits into current counter Display Time digits
	; set up the EEadr 
	; CLTNum is +2 displayed
	DECF	CLTNum,f
	MOVFW	CLTNum
	ANDLW	0x1F
	MOVWF	EEADR
	MOVFW	CLdbcdM
	CALL	EEwrt

	DECF	CLTNum,f
	MOVFW	CLTNum
	ANDLW	0x1F
	MOVWF	EEADR
	MOVFW	CLdbcdH
	CALL	EEwrt
	INCF	CLTNum,f
	INCF	CLTNum,f
	; Go and display SET and current time
	GOTO	LCDNextSet1

LCDNextSet0
	; Copy Entered digits into Current time
	MOVFW	CLdbcdH
	MOVWF	CLnbcdH
	MOVWF	CLbbcdH

	MOVFW	CLdbcdM
	MOVWF	CLnbcdM
	MOVWF	CLbbcdM

	MOVLW	0x45
	MOVWF	CLnbcdS

LCDNextSet1
        MOVLW   LCDCurRight0+7	;0xC7
        CALL    LCDwrtCmd

	MOVLW	LCDstrLset
	CALL	LCDwrtStr
	;	display set then update the time display

        GOTO	LCDredispCursorLast

		include "lcd.inc"

; the code below must be kept together.
LCDgetByte
	; we should have set up PCLATH
	; End of string sets LCDstrPtr to 0xFF so that 
	; INCFSZ can be used to jump out of loop.
	MOVLW	high $
	MOVWF	PCLATH 
	MOVFW	LCDstrPtr;
	ADDWF	PCL,f

LCDstrs
LCDstrLTime	equ	$ - LCDstrs
	DT	"Hello"
	GOTO	LCDgetByteEnd4		; now print out 4 blanks

LCDstrLTime0	equ	$ - LCDstrs
	DT	"time"
	GOTO	LCDgetByteEnd4		; now print out 4 blanks

LCDstrLblank	equ	$ - LCDstrs
	DT	"   "
	GOTO	LCDgetByteEnd6

LCDstrLOff	equ	$ - LCDstrs
	DT	"off"
	GOTO	LCDgetByteEnd3

LCDstrLOn	equ	$ - LCDstrs
	DT	"on "
	GOTO	LCDgetByteEnd3
LCDstrLOnp	equ	$ - LCDstrs
	DT	"on+"
	GOTO	LCDgetByteEnd3

LCDstrLset	equ	$ - LCDstrs
	DT	"set"
	GOTO	LCDgetByteEnd2


LCDLastStr

	TEST_STRADDLE LCDgetByte


	; Jump into these to select how many spaces printed
	; at the end, so that 9 chars are printed.
LCDgetByteEnd0
	DECF	LCDeraseCnt,f

LCDgetByteEnd1
	DECF	LCDeraseCnt,f
LCDgetByteEnd2
	DECF	LCDeraseCnt,f
LCDgetByteEnd3
	DECF	LCDeraseCnt,f
LCDgetByteEnd4
	DECF	LCDeraseCnt,f
LCDgetByteEnd5
	DECF	LCDeraseCnt,f
LCDgetByteEnd6
	CLRF	LCDstrPtr
	DECF	LCDstrPtr,f

	endif
	RETURN


;--------------------------------------------------------------------------
; Sec 4.	Subroutines, procedures and functions
;--------------------------------------------------------------------------

;--------------------------------------------------------------------------
; Sec 4.1	Button Poll Routine
;--------------------------------------------------------------------------
		
IPtimeslice
		;	----___
		;	--_____
		;
		; This reads all Port B inputs and looks for Press
		MOVFW	IPnew
		MOVWF	IPlast
		MOVFW	PORTB
		MOVWF	IPnew
		; IP last contains new setting, IPlast contains previous
		COMF	IPlast,W
		IORWF	IPnew,W
		; now force IPbuttonEvent bits to low for new pressed button
		; the service routine should set the bit to clear the event.
		ANDWF	IPbuttonEvent,F
		RETURN


;--------------------------------------------------------------------------
; Sec 4.3	Clock Chain Routine
;--------------------------------------------------------------------------
		
CLtimeslice

		; Assuming:
		; 	Crystal = 32.768 kHz
		;	Prescaller ticks at Crystal / 4 or 8192 ticks per second.
		;	Prescaller set to divide  so
		;	Tmr0 times out 4 times per second.
		;
		BTFSS	INTCON,T0IF
		RETURN

		; TMR0 timeout
		BCF	INTCON,T0IF
		BSF	DoTimeSlice,DTsS4event


		; This ticks 4 times per second, 
		; 
		DECFSZ	CLtmr4sec,f
		RETURN
		; divide by 4
		MOVLW	4
		MOVWF	CLtmr4sec
		GOTO	CLbcdS

;--------------------------------------------------------------------------
; Sec 4.4	BDC Clock Chain Routine
;--------------------------------------------------------------------------

		; BCD Counter	- Seconds
CLbcdS
		BSF	DoTimeSlice,DTsSevent
		INCF	CLnbcdS,f
		MOVF	CLnbcdS,w
		; binary add bcd digits to make 60 overflow digits.
		ADDLW	0xA6	; 10 + 6 = 16, overflows digit 6 for 10, 7 for 9, 8 for 8, 9 for 7, A for 6
		BC	CLbcdS1
		BDC	CLbcdS2
		RETURN

CLbcdS2		; unit counter carry to tens counter
		MOVFW	CLnbcdS
		; Add 1 to tens, subtract units to zero
		ADDLW	0x10 - 0x0A
		MOVWF	CLnbcdS
		RETURN

CLbcdS1		
		CLRF	CLnbcdS
		; and on to the next counter

		; BCD Counter	- Minutes

CLbcdM
		BSF	DoTimeSlice,DTsMevent	
		INCF	CLnbcdM,f
		MOVF	CLnbcdM,w
		ADDLW	0xA6	; 10 + 6 = 16, overflows digit 6 for 10, 7 for 9, 8 for 8, 9 for 7, A for 6
		BC	CLbcdM1
		BDC	CLbcdM2
		RETURN

CLbcdM2		; unit counter carry to tens counter
		MOVFW	CLnbcdM
		; Add 1 to tens, subtract units to zero
		ADDLW	0x10 - 0x0A
		MOVWF	CLnbcdM
		RETURN

CLbcdM1		
		CLRF	CLnbcdM
		; and on to the next counter

		; BCD Counter	Hours

CLbcdH
		BSF	DoTimeSlice,DTsHevent
		INCF	CLnbcdH,f
		MOVF	CLnbcdH,w
		ADDLW	0xA6		
		BC	CLbcdH1
		BDC	CLbcdH2
		; check for 24 hours
		MOVF	CLnbcdH,w
		ADDLW 0x00 - 0x24		
		SKPNZ	
		CLRF	CLnbcdH
		RETURN

CLbcdH2		; unit counter carry to tens counter
		MOVFW	CLnbcdH
		ADDLW	0x10 - 0x0A
		MOVWF	CLnbcdH
		RETURN

CLbcdH1		
		CLRF	CLnbcdH
		; and on to the next counter
		RETURN

;--------------------------------------------------------------------------
; Sec 4.5	Event Handler Routines - Timer
;--------------------------------------------------------------------------
SS4tick		;  do this 4 times a second
		BCF	DoTimeSlice,DTsS4event
		CALL	IPtimeslice
		RETURN



SStick		
		BCF	DoTimeSlice,DTsSevent
		CALL	AWts		
		; Only do serial display when Beep machine is not running as
		; beeps and serial clash
		MOVF	IPsetState,F	; test for zero
		SKPZ
		RETURN

		; Only do this if not updating display
		BSF	DoTimeSlice,DTSRwrtEvent
	        MOVLW   LCDCurRight0+7	;0xC7
	        CALL    LCDwrtCmd
	        MOVLW   ':'
	        CALL    LCDwrtChar
		MOVFW	CLnbcdS
		CALL 	LCDdispByte
		RETURN

SMtick		
		BCF	DoTimeSlice,DTsMevent
		; if an opened event has happened in the minute
		; store it.
		BTFSS	DoTimeSlice,DTopenedevent
		GOTO	SMtick0


		BCF	DoTimeSlice,DTopenedevent

		; Store the current time into the next register.
		MOVFW	OPcnt
		MOVWF	FSR
		MOVFW	CLnbcdH
		MOVWF	INDF

		INCFSZ	OPcnt,f
		MOVFW	OPcnt
		MOVWF	FSR
		MOVFW	CLnbcdM
		MOVWF	INDF

		INCFSZ	OPcnt,f
		MOVFW	OPcnt
		ADDLW	-0x50
		; Check if the end of memory
		SKPZ	
		GOTO	SMtick0	

		MOVLW	OPbufferStart
		MOVWF	OPcnt
		GOTO	SMtick0

		; Check for event
SMtick0
		MOVFW	CLTNumChk
		ANDLW	0x3F		; Restrict to 64 bytes of EEPROM
		CALL	EEread
		INCF	CLTNumChk,f
		SUBWF	CLnbcdH,w
		BNZ	SMtick1	
		; Hour found
		MOVFW	CLTNumChk
		ANDLW	0x3F		; Restrict to 64 bytes of EEPROM
		CALL	EEread
		SUBWF	CLnbcdM,w
		BNZ	SMtick1	
		; Minute Found
		CALL	STturnOn

SMtick1
		INCF	CLTNumChk,f
	
		MOVFW	CLTNumChk
		ANDLW	0x3F		; Restrict to 64 bytes of EEPROM
		CALL	EEread
		INCF	CLTNumChk,f
		SUBWF	CLnbcdH,w
		BNZ	SMtick2	
		; Hour found
		MOVFW	CLTNumChk
		ANDLW	0x3F		; Restrict to 64 bytes of EEPROM
		CALL	EEread
		SUBWF	CLnbcdM,w
		BNZ	SMtick2	
		; Minute Found
		CALL	STturnOff
SMtick2
		INCF	CLTNumChk,f
		MOVFW	CLTNumChk
		ADDLW	-0x20
		BNZ	SMtick0
		CLRF	CLTNumChk

		MOVF	IPsetState,F	; test for zero
		SKPZ
		RETURN



		; Only do this if not updating display
		; Display Hour and minute every minute.
	        MOVLW   LCDCurRight0+2	;0xC2
	        CALL    LCDwrtCmd
		MOVFW	CLnbcdH
		CALL 	LCDdispByte
	        MOVLW   ':'
	        CALL    LCDwrtChar
		MOVFW	CLnbcdM
		GOTO 	LCDdispByte


SHtick		; Turn Off On the Hour
		BCF	DoTimeSlice,DTsHevent
		RETURN

;--------------------------------------------------------------------------
; Sec 4.6	Event Handler Routine - Buttons
;--------------------------------------------------------------------------
; come here and combine the current state and the current event to index into 
; the state jump table.
;

IPsetPressed	
		BSF	IPbuttonEvent,BUTTON_SET
		MOVLW	1
		CALL 	IPsetJump
		MOVWF	IPsetState
		SWAPF	IPsetState,W
		MOVWF	BILEDPORT
		RETURN

IPnextPressed	
		BSF	IPbuttonEvent,BUTTON_NEXT
		MOVLW	0
		CALL 	IPsetJump
		MOVWF	IPsetState
		SWAPF	IPsetState,W
		MOVWF	BILEDPORT
		RETURN

IPonoffPressed	
		BSF	IPbuttonEvent,BUTTON_ONOFF
		GOTO	STtoggle
		RETURN

IPopenedPressed	
		; Debounce to the minute so that only one opened time is stored
		BSF	DoTimeSlice,DTopenedevent
		BSF	IPbuttonEvent,BUTTON_OPENED
		RETURN


;--------------------------------------------------------------------------
; Sec 4.7	Relay Control Routine
;--------------------------------------------------------------------------
STtoggle
		; Rotate STstate around three values
		; 0 = Relay Off
		; 3 = Relay On, Beep On		; The beep code has been removed
		; 2 = Relay On, Beep Off
		BTFSS	STstate,STrelay
		GOTO	STtoggleOn
		BTFSC	STstate,STbeep
		GOTO	STtoggleOff
		BSF	STstate,STbeep
		ifdef 	buildLCD
		MOVLW	LCDstrLOn
		CALL	LCDwrtStr
		endif
		RETLW	SET_NORM

STtoggleOn
		CALL	STturnOn
		RETLW	SET_NORM

STtoggleOff
		CALL	STturnOff
		RETLW	SET_NORM


STturnOn	; Turn on Radio and Tape to record forecast

	ifdef 	buildLCD
		MOVLW	LCDstrLOnp
		CALL	LCDwrtStr
		movfw	SRin
		CALL    LCDwrtChar
	endif

	; Start the AIWA
	CALL	AWstart

	BSF	STstate,STrelay
	BCF	STstate,STbeep
;	BSF	PORTB,RELAY		; This is now down in Awai.Inc
	RETURN


STturnOff	; Turn off Radio and Tape to record forecast

	ifdef 	buildLCD
		MOVLW	LCDstrLOff
		CALL	LCDwrtStr

		movfw	SRin
		CALL    LCDwrtChar

	endif
	
	; Stop the AIWA
	CALL	AWstop
	BCF	STstate,STrelay
	BCF	STstate,STbeep
;	BCF	PORTB,RELAY		; This is now done in Awai.inc
	RETURN


include "awai.inc"

;--------------------------------------------------------------------------
; Sec 4.8	EEprom Routines
;--------------------------------------------------------------------------

EEread		
		; w contains address
		BCF	STATUS,RP0
		MOVWF	EEADR
		BSF	STATUS,RP0
		BSF	EECON1,RD
		BCF	STATUS,RP0
		MOVFW	EEDATA
		RETURN

EEwrt		; blocking, EEADR and EEDATA set up.
		BCF	STATUS,RP0
		MOVWF	EEDATA
		BSF	STATUS,RP0
		BCF	INTCON,GIE
		BCF	EECON1,EEIF
		BSF	EECON1,WREN
		MOVLW	0x55
		MOVWF	EECON2
		MOVLW	0xAA
		MOVWF	EECON2
		BSF	EECON1,WR
		; Now wait for EEwrt to finish write
EEwaitForWrt	; block until EEPROM write has finished.
		BCF	EECON1,WREN	; disable EROM
		BTFSS	EECON1,EEIF
		GOTO	EEwaitForWrt
		BCF	EECON1,EEIF	; EEPROM finished
		BCF	STATUS,RP0
		BSF	INTCON,GIE

		RETURN
		

		
;--------------------------------------------------------------------------
; Sec 4.9	Serial Out
;--------------------------------------------------------------------------
;				
;		@ 8192 instructions a second, 
;		for 1200 baud, each bit = 6.82666 instructions
;		
;		---_01234567P---- 
;		-> Start     Stop
;		The output signal is inverted by the driver transistor
;
;		8192/	instrns	cumulative	% er
;		1200
;
;		0.000		0	
;	start	6.827	7	7	102.54%
;	0	13.653	7	14	102.54%
;	1	20.480	7	21	102.54%
;	2	27.307	6	27	98.88%
;	3	34.133	7	34	99.61%
;	4	40.960	7	41	100.10%
;	5	47.787	7	48	100.45%
;	6	54.613	7	55	100.71%
;	7	61.440	6	61	99.28%
;	8	68.267	7	68	99.61%
;	end	75.093	7	75	99.88%
;					

;#Is the output inverted? 
;# Normally the UART OP is inverted by the Line driver
;# but as design does not uses a driver invert the bits
RS232opHigh	MACRO	
		BCF	PORT_RS232,RS232OUT
	endm

RS232opLow	MACRO	
		BSF	PORT_RS232,RS232OUT
	endm

RS232opBit	MACRO   bit
		BTFSC	SRout,bit		; 1 if false, 2 if skip
		GOTO	SRtxCharA#v(bit)	; 	t+1 instructions
		NOP			;t+2
		;BSF	PORT_RS232,RS232OUT		;t+3
		RS232opLow
		GOTO	SRtxCharA#v(bit)a	;t+4
SRtxCharA#v(bit)	;BCF	PORT_RS232,RS232OUT		;	t+3
		RS232opHigh

		NOP			;	t+4
		NOP			;	t+5
SRtxCharA#v(bit)a	NOP			;t+6	t+6
	endm

	ifdef buildMacroRS232
SRtxChar
		;BSF	PORT_RS232,RS232OUT		;		t=0
		RS232opLow
;		MOVLW	'U'
		MOVWF	SRout

		NOP			;		t+1
		NOP			;		t+2		
;		NOP			;		t+3
		; These take 4 instruction cyles
		; t=0
		NOP


		RS232opBit	0
		RS232opBit	1
		RS232opBit	2
		RS232opBit	3
		RS232opBit	4
		RS232opBit	5
		RS232opBit	6
		RS232opBit	7
		; Stop Bit
		RS232opHigh
;		BCF	PORT_RS232,RS232OUT

		NOP
;		NOP
;		NOP
;		NOP
;		NOP
;		NOP
		RETURN
	endif

SRwrtHexNibble	; currently only displays 0..9, A..F
	ANDLW	0x0F
	ADDLW	0x06	; is it A..F, if so trigger a digit overflow
	SKPNDC
	ADDLW	7; subtract 10, then add 'A'-'0'
	ADDLW	0x30-6	; Subtract extra 6 added to cause DC
	MOVWF	SRout
	GOTO	SRtxChar
	
;--------------------------------------------------------------------------
; Sec 4.9	Serial Out Display Procedure
;--------------------------------------------------------------------------

SRwrtTime
  ifdef buildSRwrt
	MOVLW	0x0D
	CALL	SRtxChar	
	MOVLW	0x0A
	CALL	SRtxChar	

	; Using the Input state machines construct a suitable display string
	; out put to the serial port.
	MOVFW	SRin
	CALL	SRtxChar	

	MOVLW	'*'
	CALL	SRtxChar	

	MOVLW	' '
	CALL	SRtxChar	


	MOVF	IPsetState,f
	BNZ	SRwrtBBcdTime
		
	; This displays the time and status of the Radio

	MOVLW	't'
	CALL	SRtxChar	

	SWAPF	CLnbcdH,W
	CALL	SRwrtHexNibble	
	MOVFW	CLnbcdH
	CALL	SRwrtHexNibble	
	MOVLW	':'
	CALL	SRtxChar	
	SWAPF	CLnbcdM,W
	CALL	SRwrtHexNibble	
	MOVFW	CLnbcdM
	CALL	SRwrtHexNibble	
	MOVLW	':'
	CALL	SRtxChar	
	SWAPF	CLnbcdS,W
	CALL	SRwrtHexNibble	
	MOVFW	CLnbcdS
	CALL	SRwrtHexNibble	
	MOVLW	':'
	CALL	SRtxChar	
	MOVLW	'o'
	CALL	SRtxChar	

	
	TEST_STRADDLE JMPlcdT8	

	MOVLW	high $
	MOVWF	PCLATH
	
	MOVFW	STstate
	ANDLW	0x03
	ADDWF 	PCL,F
	GOTO	SRwrtTimeDispT0	; off		
	GOTO	SRwrtTimeDispT3 ; on beep
	GOTO	SRwrtTimeDispT1		
	GOTO	SRwrtTimeDispT2	; on, no beep	

JMPlcdT8

SRwrtTimeDispT0	
	MOVLW	'f'
	CALL	SRtxChar	
	MOVLW	'f'
	GOTO	SRwrtBBcdTime2
		
SRwrtTimeDispT1
	MOVLW	'1'
	CALL	SRtxChar	
	GOTO	SRwrtBBcdTime2

SRwrtTimeDispT2	
	MOVLW	'n'
	GOTO	SRwrtBBcdTime2

SRwrtTimeDispT3	
	MOVLW	'n'
	CALL	SRtxChar	

	MOVLW	' '
	CALL	SRtxChar	

	MOVLW	'b'
	CALL	SRtxChar	

	MOVLW	'e'
	CALL	SRtxChar	

	MOVLW	'e'
	CALL	SRtxChar	

	MOVLW	'p'
	GOTO	SRwrtBBcdTime2


SRwrtBBcdTime
	; During time setting, now write out the register value, or now time

	BNZ	SRwrtBBcdTime1
	BTFSS	CLTNum,7
	GOTO	SRwrtBBcdTime1
	; current 	
	MOVLW	' '
	CALL	SRtxChar	
	MOVLW	'n'
	CALL	SRtxChar	
	MOVLW	'o'
	CALL	SRtxChar	
	MOVLW	'w'
	CALL	SRtxChar	
	GOTO	SRwrtBBcdTime3
SRwrtBBcdTime1
	BTFSC	CLTNum,1	; test second bit 
	GOTO	SRwrtBBcdTime1a
	MOVLW	'O'
	CALL	SRtxChar	
	MOVLW	'f'
	CALL	SRtxChar	
	MOVLW	'f'
	CALL	SRtxChar	
	MOVLW	'@'
	CALL	SRtxChar	
	GOTO	SRwrtBBcdTime3
SRwrtBBcdTime1a
	MOVLW	' '
	CALL	SRtxChar	
	MOVLW	'O'
	CALL	SRtxChar	
	MOVLW	'n'
	CALL	SRtxChar	
	MOVLW	'@'
	CALL	SRtxChar	
	GOTO	SRwrtBBcdTime3
SRwrtBBcdTime3
	; display leaves Cursor right of ? below
	;		on@#1:H H:M M:? 
	; These are spaced 2 Chars aprt so that a copy of DispState 
	; can be used to backspace the cursor over the digit being set.
	MOVLW	'#'
	CALL	SRtxChar	
	RRF	CLTNum,W
	CALL	SRwrtHexNibble	
	MOVLW	':'
	CALL	SRtxChar	
	SWAPF	CLdbcdH,W
	CALL	SRwrtHexNibble	
	MOVLW	' '
	CALL	SRtxChar	
	MOVFW	CLdbcdH
	CALL	SRwrtHexNibble	

	MOVLW	':'
	CALL	SRtxChar	
	SWAPF	CLdbcdM,W
	CALL	SRwrtHexNibble	
	MOVLW	' '
	CALL	SRtxChar	
	MOVFW	CLdbcdM
	CALL	SRwrtHexNibble	
	MOVLW	':'
	CALL	SRtxChar	
	MOVLW	'?'
	CALL	SRtxChar	
	MOVLW	' '
	CALL	SRtxChar	

	; Now reposition Cursor.
	; cntl-H moved the cursor left.
	; :H H:M M:?

	; use a jump table to position the cursor on the digit being input
	; Use the DispState to backspace the 
	; cursor over on@#1:H H:M M?
	;
	MOVFW	DispState	; dispstate is 0,2,4,6,8 
	ADDLW	-11		
	MOVWF	SRloopCnt
SRwrtLoop
	MOVLW	08		; Backspace cursor
	CALL	SRtxChar	
	INCFSZ	SRloopCnt,F
	GOTO	SRwrtLoop

SRwrtBBcdTime2

	CALL	SRtxChar	
	BCF	DoTimeSlice,DTSRwrtEvent

  endif
	RETURN


;--------------------------------------------------------------------------
; Sec 5.0	Interrupt Routines
;--------------------------------------------------------------------------

Intrtn	

		movwf   w_temp            ; save off current W register contents
		movf	STATUS,w          ; move status register into W register
		movwf	status_temp       ; save off contents of STATUS register
		; Which Interupt ?
		; No Context Save in these routines.

		; RS232 1200 baud serial code 

TurboSRinA
		; for the 1200 baud serial input, each bit takes 6.83 cycles(8192/1200)
		; We need to sample 1.5 bits (  10.245 cycles in )
		;RS232 output pin must be RB0 ( or RB7 with mods )

TurboSRinA1
		RRF	PORTB,W	; shift LSB into Carry
		RRF	SRin,f
		goto	$+1	; wants 2 cycle delay
		DECFSZ	SRcnt,f
		GOTO	TurboSRinA1
		; now set up the counter for next time
		MOVLW	8
		MOVWF	SRcnt

		; The SRin needs to be inverted
		COMF	SRin,f
		MOVFW	SRin
		; use the inverted format and add '2'
		ADDLW	-'1'
		SKPNZ
		BCF	IPbuttonEvent,BUTTON_SET	; On BUTTON_SET Press
		ADDLW	'1'-'0'
		SKPNZ
		BCF	IPbuttonEvent,BUTTON_NEXT	; On BUTTON_NEXT Press

IntrtnEnding
		BCF 	INTCON,INTF
		movf    status_temp,w     ; retrieve copy of STATUS register
		movwf	STATUS            ; restore pre-isr STATUS register contents
		swapf   w_temp,f
		swapf   w_temp,w          ; restore pre-isr W register contents
		retfie                    ; return from interrupt


CodeEnd		goto	CodeEnd







;--------------------------------------------------------------------------
; Program End
;--------------------------------------------------------------------------

;--------------------------------------------------------------------------
; EEPROM data
;--------------------------------------------------------------------------

 
		ORG	0x2100

		; Time for the four Shipping forecasts
		DE	0x00,0x45  ; 0
		DE	0x00,0x58
		DE	0x05,0x33  ; 1
		DE	0x05,0x47
		DE	0x11,0x59  ; 2
		DE	0x12,0x05  
		DE	0x17,0x54  ; 3
		DE	0x18,0x01

		; first 2 on and off are general ones
		DE	0x04,0x30  ; 4
		DE	0x04,0x35
		DE	0x25,0x00  ; 5
		DE	0x25,0x00

		DE	0x25,0x00  ; 4
		DE	0x25,0x00
		DE	0x25,0x00  ; 5
		DE	0x25,0x00
		DE	0x25,0x00  ; 4
		DE	0x25,0x00
		DE	0x25,0x00  ; 5
		DE	0x25,0x00



		; Last four are general purpose
		DE	0x25,0x00  ; 6
		DE	0x25,0x00
		DE	0x25,0x00  ; 7
		DE	0x25,0x00

	cblock
LASTcont
	endc

               END


; 1 N	31 4E 01001110
; 2 M	32 4D 01001101
; 3 K
